@@ -6,7 +6,8 @@
 # 20210815 | Michael Tatton | Initial version
 # 20210820 | Michael Tatton | v0.0.8
 # 20210821 | Michael Tatton | v0.0.9
-# 20210822 | Michael Tatton | v0.1.0
+# 20210821 | Michael Tatton | v0.1.0
+# 20210822 | Michael Tatton | v0.1.1
 #
 
 import os
@@ -17,8 +18,7 @@ import traceback
 from ipykernel.kernelbase import Kernel
 from ipykernel.kernelapp import IPKernelApp
 import psycopg2
-
-# import signal
+#import signal
 
 str_kernel = "sqlpk"
 __version__ = "0.1.0"
@@ -61,6 +61,8 @@ class DBConnection:
 
     def connect(self, constr):
         try:
+            if self.connected is True:
+                self.disconnect()
             self.constr = constr
             self.con = psycopg2.connect(**self.constr)
             log("-- CONNECTED TO DATABASE")
@@ -88,7 +90,7 @@ class DBConnection:
                     cur.execute(qry)
                 except Exception as e:
                     self.con.rollback()
-                    rows = [[str(e)]]
+                    rows=[[str(e)]]
                     log("-- QUERY EXECUTION ERROR: " + str(e))
                     error_ind = True
                 # TRY TO FETCH DATA
@@ -99,12 +101,14 @@ class DBConnection:
                 except Exception as e:
                     log("-- NO FETCH : " + str(e))
                 # COMMIT WHEN NO ERROR IS PRESENT
-                if error_ind is False and data_ind is False:
+                #log("-- COMMIT YN: " + str(error_ind) + " " + str(data_ind))
+                if error_ind is False:
                     self.con.commit()
-                    rows = [["OK"]]
+                if error_ind is False and data_ind is False:
+                    rows=[["OK"]]
                 # IF DATA AND NO ERROR GET HEADER
                 if data_ind is True and cur.description:
-                    # log(str(rows))
+                    #log(str(rows))
                     for cn in cur.description:
                         hdr.append(cn[0])
                     df.append(hdr)
@@ -129,19 +133,21 @@ class DBConnection:
             return (retval, rows)
 
     def disconnect(self):
-        self.connected = False
+        self.con.commit()
         self.con.close()
         log("-- DISCONNECTED")
+        self.connected = False
 
     def __del__(self):
         self.disconnect()
 
-
 class SQLpKernel(Kernel):
 
     implementation = str_kernel
     implementation_version = __version__
     dbcon = None
+    #html_output = True
+    html_output = False
 
     con = None
     constr = {
@@ -152,6 +158,7 @@ class SQLpKernel(Kernel):
         "password": "",
     }
 
+
     @property
     def language_version(self):
         return ""
@@ -166,7 +173,7 @@ class SQLpKernel(Kernel):
         "file_extension": ".sql",
     }
 
-    # def handler(signum, frame):
+    #def handler(signum, frame):
     #    log("- - - - - - - - - - - - - SIGINT || SIGTERM")
 
     def __init__(self, **kwargs):
@@ -175,11 +182,27 @@ class SQLpKernel(Kernel):
             self.dbcon = self.constr
             self.con = DBConnection(self.dbcon)
             log("__init__")
+            #self.html_output = True
+            self.html_output = False
         except Exception as e:
             log("-- KERNEL INIT PROBLEM: " + str(e))
             log(traceback.format_exc())
-        # signal.signal(signal.SIGINT, self.handler)
-        # signal.signal(signal.SIGTERM, self.handler)
+        #signal.signal(signal.SIGINT, self.handler)
+        #signal.signal(signal.SIGTERM, self.handler)
+
+    def send_message_html(self, html ):
+        try:
+            message = {"data": { "text/html": html }, "metadata": {}, }
+            self.send_response(self.iopub_socket, "display_data", message)
+            return {
+                "status": "ok",
+                "execution_count": self.execution_count,
+                "payload": [],
+                "user_expressions": {},
+            }
+        except Exception as e:
+            log("-- SEND HTML MESAGE ERROR: " + str(e))
+            log(traceback.format_exc())
 
     def send_message(self, msg):
         try:
@@ -195,18 +218,37 @@ class SQLpKernel(Kernel):
             log("-- SEND MESAGE ERROR: " + str(e))
             log(traceback.format_exc())
 
+    def conv_data_to_html(self, res):
+        msg = ""
+        try:
+            if self.con.cols and self.con.rows:
+                 msg = "<table>"
+                 for row in res:
+                     msg += "<tr>"
+                     for cell in row:
+                         msg += "<td>" + str(cell) +"</td>"
+                     msg += "</tr>"
+                 msg += "</table>"
+            else:
+                msg = "No data"
+        except Exception as e:
+            log("-- CONV DATA TO HTML ERROR: " + str(e))
+            log(traceback.format_exc())
+
+        return(msg)
+
     def save_data_to_csv(self, file_name):
-        msg = None
+        msg=None
         if self.con.cols and self.con.rows:
-            try:
-                with open(file_name + ".csv", "w") as out:
-                    csv_out = csv.writer(out)
-                    csv_out.writerow(self.con.cols)
-                    for row in self.con.rows:
-                        csv_out.writerow(row)
-                    msg = "Written " + file_name + ".csv"
-            except Exception as e:
-                msg = "Error during csv export " + str(e)
+             try:
+                 with open(file_name+'.csv','w') as out:
+                     csv_out = csv.writer(out)
+                     csv_out.writerow(self.con.cols)
+                     for row in self.con.rows:
+                         csv_out.writerow(row)
+                     msg = "Written "+file_name+".csv"
+             except Exception as e:
+                 msg="Error during csv export "+str(e)
         else:
             msg = "No data to save"
         self.send_message(msg)
@@ -229,7 +271,7 @@ class SQLpKernel(Kernel):
                     if magics["dbcon"]:
                         self.dbcon = str(magics["dbcon"])
                         self.constr = self.dbcon
-                        # log(str(self.dbcon))
+                        #log(str(self.dbcon))
                     else:
                         # status, res = self.con.qry2df(code, self.constr)
                         status, res = self.con.qry2df(code)
@@ -238,13 +280,19 @@ class SQLpKernel(Kernel):
 
                 if res:
                     if len(res) > 1:
-                        ret = tabulate(res, headers="firstrow")
+                        if self.html_output is True:
+                            ret = self.conv_data_to_html(res)
+                        else:
+                            ret = tabulate(res, headers="firstrow")
                     else:
                         ret = str(res[0][0])
             else:
                 ret = "NOT CONNECTED"
             if ret:
-                self.send_message(ret)
+                if self.html_output is True:
+                    self.send_message_html(ret)
+                else:
+                    self.send_message(ret)
         except Exception as e:
             log("-- ERROR IN DO_EXECUTE: " + str(e))
             log(traceback.format_exc())
@@ -260,18 +308,26 @@ class SQLpKernel(Kernel):
                     # log("-- dbconln magics: " + dbconln[0:5])
                     if dbconln[0:5] == "dbcon":
                         if len(dbconln) > 6:
-                            magics["dbcon"] = dbconln[6:]
-                            self.dbcon = json.loads(magics["dbcon"])
-                            self.constr = self.dbcon
-                            log("-- TYRING TO CONNECT WITH: " + str(self.constr))
-                            conret = self.con.connect(self.constr)
-                            log("-- MAGICS DBCON CONNECTED: " + str(self.con.connected))
-                            self.send_message(conret)
+                              magics["dbcon"] = dbconln[6:]
+                              self.dbcon = json.loads(magics["dbcon"])
+                              self.constr = self.dbcon
+                              log("-- TYRING TO CONNECT WITH: " + str(self.constr))
+                              conret = self.con.connect(self.constr)
+                              log("-- MAGICS DBCON CONNECTED: " + str(self.con.connected))
+                              self.send_message(conret)
+                    elif dbconln[0:5] == "otext":
+                        self.html_output = False
+                        self.send_message("Output mode set to text")
+                        magics["noexec"] = True
+                    elif dbconln[0:5] == "ohtml":
+                        self.html_output = True
+                        self.send_message("Output mode set to html")
+                        magics["noexec"] = True
                     elif dbconln[0:5] == "dsave":
-                        fname = "tmp"
+                        fname="tmp"
                         if len(dbconln) > 6:
-                            fname = dbconln[6:]
-                        log("-- SAVING DATA TO FILE")
+                            fname = dbconln[6:] 
+                        log("-- SAVING DATA TO FILE")       
                         self.save_data_to_csv(fname)
                         magics["noexec"] = True
                     elif dbconln[0:5] == "csave":
@@ -311,5 +367,4 @@ class SQLpKernel(Kernel):
             os.remove(file)
         os.remove(self.master_path)
 
-
 IPKernelApp.launch_instance(kernel_class=SQLpKernel)
